@@ -1,4 +1,4 @@
-#! python3.8
+#! python3.9
 # -*- coding: utf-8 -*-
 #
 # This Python script parses the Unicode data files and generates the C files
@@ -1074,7 +1074,8 @@ def generate_all_cases(unicode_data, c_file):
     max_columns = max(len(value) for value in others_dict)
 
     max_width = max(len(str(item)) for value in others_dict for item in value)
-    fmt = '    {{' + ', '.join(['{:%d}' % max_width] * max_columns) + '}},\n'
+    fmt = '    {{{:%d}, {{' % max_width + ', '.join(['{:%d}' % max_width] *
+      (max_columns -1)) + '}}}},\n'
 
     lines = []
 
@@ -1245,8 +1246,8 @@ def generate_full_case_folding(unicode_data, c_file):
     max_width = max(len(str(item)) for value in value_dict for item in value)
     rows = [(value + (0, ) * max_folded)[ : max_folded] for value in
       sorted(value_dict, key=value_dict.get)]
-    fmt = ('    {{' + ', '.join(['{:%d}' % max_width] * max_folded) +
-      '}},\n').format
+    fmt = ('    {{{{' + ', '.join(['{:%d}' % max_width] * max_folded) +
+      '}}}},\n').format
     lines = []
 
     for row in rows:
@@ -1397,7 +1398,7 @@ def generate_code(unicode_data, tools_folder):
             val_list = list(unique(property['values'].values(), key=id))
 
             for value in sorted(val_list, key=lambda val: val['id']):
-                valueset.append(tuple(value['names']))
+                valueset.append((value['id'], tuple(value['names'])))
 
             valueset_id = valueset_dict.setdefault(tuple(valueset),
               len(valueset_dict))
@@ -1476,10 +1477,10 @@ RE_PropertyValue re_property_values[] = {
 
         for valset, valset_id in sorted(valueset_dict.items(), key=lambda pair:
           pair[1]):
-            if valset_id == gc_valset_id:
-                valset = sorted(valset, key=make_key)
+            for val_id, names in valset:
+                if valset_id == gc_valset_id:
+                    names = sorted(names, key=make_key)
 
-            for val_id, names in enumerate(valset):
                 for name in names:
                     c_file.write('''    {{{:4}, {:2}, {:3}}}, /* {} */\n'''.format(strings_dict[munge(name)],
                       valset_id, val_id, munge(name)))
@@ -1699,7 +1700,7 @@ typedef RE_UINT32 (*RE_GetPropertyFunc)(RE_UINT32 codepoint);
         h_file.write('int re_get_full_case_folding(RE_UINT32 codepoint, RE_UINT32* folded);\n')
 
 # Whether to update the Unicode data files from the Unicode website.
-UNICODE_VERSION = '13.0.0'
+UNICODE_VERSION = '14.0.0'
 
 # The URL from which the Unicode data files can be obtained.
 unicode_data_base = 'http://www.unicode.org/Public/UNIDATA/'
